﻿using System;
using System.IO;
using System.Text;
using System.Data;
using HitsujiLibrary201004.Utils.Data;
using HitsujiLibrary201004.ObserverPattern.Subject;
using HitsujiLibrary201004.ObserverPattern.Observer;
using HealthyHitsujiWpf.Model.Content;
using HealthyHitsujiWpf.Model.DataBase;
using HealthyHitsujiWpf.ObserverPattern.Subject;
using Microsoft.Win32;

namespace HealthyHitsujiWpf.ObserverPattern.Observer
{
    public class OutPutMonthDataAsCsvFileObserver : _IObserver
    {

        //出力対象のカラム指定のためのデリゲート
        private Action<DataTable> targetColumnSelectionDelegate_;
        public Action<DataTable> targetColumnSelectionDelegate
        {
            get { return targetColumnSelectionDelegate_; }
            set { targetColumnSelectionDelegate_ = value; }
        }
        //出力対象のフィールド生成のためのデリゲート
        private Action<DataRow, _IContentModel> targetFieldSelectionDelegate_;
        public Action<DataRow, _IContentModel> targetFieldSelectionDelegate
        {
            get { return targetFieldSelectionDelegate_; }
            set { targetFieldSelectionDelegate_ = value; }
        }

        #region _IObserver メンバ

        public void doMyRole(_ASubject _ASubject)
        {
            //カレントディレクトリ取得
            String currentDirectory = System.IO.Directory.GetCurrentDirectory();

            //ダイアログ設定
            var saveFileDialog = new Microsoft.Win32.SaveFileDialog();
            saveFileDialog.InitialDirectory = System.Environment.GetFolderPath(System.Environment.SpecialFolder.Personal);
            saveFileDialog.Filter = "CSVファイル(*.csv)|*.csv";
            saveFileDialog.Title = "CSVデータ出力先を指定してください。";
            saveFileDialog.OverwritePrompt = true;

            //ファイル名 
            saveFileDialog.FileName = "";

            if ((bool)saveFileDialog.ShowDialog())
            {
                var outPutFullPath = saveFileDialog.FileName; 
                
                //CSVファイル生成処理開始
                var subject = (GetSelectedMonthDataSubject)_ASubject;
                var selectedDateInfo = (SelectedDateModel)subject.models["SelectedDateModel"];
                var monthContentsinfo = (MonthContentsModel)subject.models["MonthContentsModel"];

                var year = selectedDateInfo.SelectedDate.Year;
                var month = selectedDateInfo.SelectedDate.Month;

                var tempDataSet = new HDataSet();
                var myTable = new DataTable();
                tempDataSet.DataSetName = MonthDataBaseColumnModel.dataSetName_;
                myTable.TableName = MonthDataBaseColumnModel.dataTableName_;
                tempDataSet.Tables.Add(myTable);

                //カラム設定
                myTable.Columns.Add(new DataColumn(MonthDataBaseColumnModel.day_));
                targetColumnSelectionDelegate_(myTable);

                //フィールドデータ設定
                for (int dayId = 1; dayId <= monthContentsinfo.Count; dayId++)
                {
                    DateContentsModel d = monthContentsinfo.getField(dayId);
                    var myRow = myTable.NewRow();
                    myRow[MonthDataBaseColumnModel.day_] = dayId.ToString();
                    targetFieldSelectionDelegate_(myRow, d);
                    myTable.Rows.Add(myRow);
                }

                tempDataSet.writeCsv(outPutFullPath,
                                     MonthDataBaseColumnModel.dataTableName_,
                                     ',',
                                     false);

                //一応参照を破棄
                tempDataSet.Clear();
                myTable = null;
                tempDataSet = null;
            }
            // カレントディレクトリを元に戻す 
            System.IO.Directory.SetCurrentDirectory(currentDirectory);
            
        }

        #endregion

        #region _IOutPutCsvFileObserver メンバ

        private StringBuilder filePath_;
        public void setOutPutFilePath(StringBuilder filePath)
        {
            filePath_ = filePath;
        }

        #endregion
    }
}
